@@ -646,7 +646,7 @@ func (h *BufPane) ShowKeyCmd(args []string) {
 		InfoBar.Error(err)
 		return
 	}
-	if action, ok := config.Bindings[event.Name()]; ok {
+	if action, ok := config.Bindings["buffer"][event.Name()]; ok {
 		InfoBar.Message(action)
 	} else {
 		InfoBar.Message(args[0], " has no binding")
@@ -820,7 +820,7 @@ func (h *BufPane) ReplaceCmd(args []string) {
 		searchLoc := h.Cursor.Loc
 		var doReplacement func()
 		doReplacement = func() {
-			locs, found, err := h.Buf.FindNext(search, start, end, searchLoc, true, !noRegex)
+			locs, found, err := h.Buf.FindNext(search, start, end, searchLoc, true, true)
 			if err != nil {
 				InfoBar.Error(err)
 				return
